/******************** (C) COPYRIGHT 2006 STMicroelectronics ********************
* File Name          : eeprom.c
* Author             : MCD Application Team
* Date First Issued  : 12/06/06 : Eprom Emulation Version 1.0
* Description        : This file provides all routines used in Eprom emulation.
********************************************************************************
* History:
* 12/06/06 : Eprom Emulation Version 1.0
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "eeprom.h"
#include "asc.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/*******************************************************************************
* Function Name  : EepromFormat
* Description    : Erases sector1 & sector2 and declares sector1 as VALID_SECTOR
* Input          : None.
* Output         : None.
* Return         : 0 on success or Flash error code on failure
* Attention      : this function erases both sector 1 and 2
*******************************************************************************/
u16 EepromFormat(void)
{
    u16 status,status1;
    u32 address;

    address = SECTOR1_BASE_ADDRESS;
    /* Erase sector1 */
    FlashSectorErase(0,SECTOR1);
    /* Wait until erase operation is completed */
    /* This function returns 0 if erase is successful and Flash error code if failed */
    status =  FlashWait(FLASH_DELAY_5SEC);
    /*if erase operation is  successful */
    if (status == 0)
    {
        /* Declare sector1 as VALID_SECTOR: write VALID_SECTOR at sector1 base address */
        status1 = FlashProgramWord(address,VALID_SECTOR);
        /* if program operation was failed, a Flash error code is returned */
        if (status1 != 0) return status1;
    }
    /* if erase operation is not successful so return Flash error code */
    else
    {
        return status;
    }
    /* Erase sector2 */
    FlashSectorErase(0,SECTOR2);
    /* Wait until erase operation is complete*/
    status =  FlashWait(FLASH_DELAY_5SEC);
    /* Return status which is 0 on success and Flash error code on failure */
    return status;
}/* End EepromFormat */

/*******************************************************************************
* Routine:  WriteVariable
* Purpose:  Writes/upadtes  variable data in EEPROM.
* Input  :  - index: variable index in the array T
*           - T: variables' array
*           - data: 32 bit data to be written
* Output :   None.
* Return : 	0 on success or error code on failure .
/******************************************************************************/

u16 WriteVariable(u8 index, u32 *T,u32 data)
{
    u16 status;
    status = WriteVerifyVariableFull(index,T,data);
    /* When there isn't enough memory space for the new variable value, the new data
    variable + the last variables' updates are transferred to a blank sector */
    if (status == VARIABLE_FULL)
    {
        /* Perform sector transfer*/
        status = EepromSectorTransfer(index, T, data);
    }
    return status;
}/* End WriteVariable */

/*******************************************************************************
* Routine: ReadVariable
* Purpose: Reads the last stored variable data
* Input  : - index: variable index in the array T.
*          - T: variable array
* Output : None.
* return : 32 bit data on success or error code on failure
*******************************************************************************/
u32 ReadVariable(u8 index, u32 *T)
{
    u8 valid_sector;
    u32 address, base_address,Read_data, variable_end_address;

    /* Get active sector  for READ operation */
    valid_sector = FindValidSector(READ_FROM_VALID_SECTOR);
    if (valid_sector == No_Valid_Sector) return  No_Valid_Sector;
    /* Get the last location in the space memory allocated for the variable */
    if (index != max_varia-1) /* If variable isn't the last one in the sector */
    {
        variable_end_address = T[index + 1] + (valid_sector - 0x1) * SECTOR_SIZE;
    }
    else  /* If variable is the last one in the sector */
    {
        variable_end_address = SECTOR1_END_ADDRESS;
        variable_end_address+= (valid_sector - 0x01) * SECTOR_SIZE;
    }

    /* Get variable base address in the valid sector  */
    base_address = (u32)(T[index] + ((valid_sector - 0x01) * SECTOR_SIZE));
    address = base_address;
    do
    {
        /* Go through the variable beginning with its base address until finding a blanck location,.
        The data in the previous location is returned  */
        /* Read address content */
        Read_data =  FlashRead(address);
        /* If location is not blank */
        if (Read_data != 0xFFFFFFFF)
        {
            /* Increment + 4 for second location */
            address = address + 4;
        }/* End if */
        /* Read_data = 0xFFFFFFFF */
        else
        {
            /*0xFFFFFFFF is a data and address is not a blank location*/
            if (FlashRead(address + 4) == 0x00000000)
            {
                /* Increment +8: 4 for 0xFFFFFFFF and 4 for 0x00000000 */
                address = address + 8;
            }
            else if (FlashRead(address +4) == 0xFFFFFFFF)
            {
                if (address != base_address)
                {
                    /* case of last update = 0xFFFFFFFF */
                    if ((FlashRead(address - 8) == 0xFFFFFFFF)&&(FlashRead(address - 4) == 0x00000000))
                    {
                        Read_data = 0xFFFFFFFF;
                        break;
                    }
                    else
                    {
                        Read_data =  FlashRead(address - 4);
                        break;
                    }
                }
                else
                    /* address = variable base address so the variable is EMPTY */
                {
                    Read_data = VARIABLE_EMPTY;
                    break;
                }
            }/* End else if	 FLASH_READ(address + 4) == 0xFFFFFFFF */
        } /* End else  Read_data = 0xFFFFFFFF */
    } while (address < variable_end_address); /* End do */

    return Read_data;

}/*End ReadVariable */

/*******************************************************************************
* Routine:  FindValidSector
* Purpose:  Find valid sector for write or read operation
* Input  :  a byte indicating whether we search valid sector for read from or
*           write into( READ_FROM_VALID_SECTOR or WRITE_IN_VALID_SECTOR)
* Output : None.
* return : 	 Sector number
*******************************************************************************/
u8 FindValidSector(u8 operation)
{
    u32 address, address1,STATUS0, STATUS1;
    /* Check sector 0 for status */
    address = SECTOR1_BASE_ADDRESS;
    STATUS0 = FlashRead(address);
    /* Check sector 1 for status */
    address1 = SECTOR2_BASE_ADDRESS;
    STATUS1 = FlashRead(address1);
    /* write or read: either we want to write in or read from the suitable sector */
    switch (operation)
    {
    case WRITE_IN_VALID_SECTOR:	  /* write operation */
        if ( STATUS1 == VALID_SECTOR) /* sector1 valid	*/
        {
            /* sector0 receiving data */
            if ( STATUS0 == RECEIVE_DATA)  return (SECTOR1);
            else return (SECTOR2);
        }
        else if ( STATUS0 == VALID_SECTOR)/* sector0 valid */
        {
            /* sector1 receiving data */
            if ( STATUS1 == RECEIVE_DATA) return (SECTOR2);
            else return (SECTOR1);
        }
        else return No_Valid_Sector;
    case READ_FROM_VALID_SECTOR: /* read operation */
        /* sector0 valid */
        if ( STATUS0 == VALID_SECTOR)  	return (SECTOR1);
        /* sector1 valid */
        else if ( STATUS1 == VALID_SECTOR)  return (SECTOR2);
        else return No_Valid_Sector ;

    default:
        return (SECTOR1);
    }/* End switch */

} /* End  FindValidSector */

/*******************************************************************************
* Routine:  FLashRead
* Purpose:  Reads data from ST10F27x flash.
* Input  :  A 32 bit address.
* Output :  None.
* return :  Data at the given address.
*******************************************************************************/
u32 FlashRead(u32 Address)
{
    u32 FlashData;
    if (SYSCON & 0x1000)	/* if ROMS1 set */
    {
        if ( (0x0000 <= Address) && (Address <0x08000)) Address = Address | 0x00010000;
    }
    /* Read data from flash  */
#ifdef Tasking
    FlashData = *( _huge u32 *)Address;
#endif
#ifdef Keil
    FlashData = *(u32 huge *)Address;
#endif
    return FlashData;
} /* End FlashRead */

/*******************************************************************************
* Routine:  WriteVerifyVariableFull
* Purpose:  Writes  variable in EEPROM.
* Input  :  - index: variable index in the array (0 for A, 1 for B, 2 for C, etc...)
*           - T: variables' Array
*           - Data: 32 bit data to be written
* Output:    None.
* Return: 	 Returns 0 on success OR 0x80 if variable is full OR Flash error code
*******************************************************************************/
u16 WriteVerifyVariableFull(u8 index, u32 *T,u32 Data)

{
    u8 valid_sector;
    s16 status, status1, status2, status3= -1;
    u32 address,variable_end_address;
    /* Get valid sector for write operation */
    valid_sector = FindValidSector(WRITE_IN_VALID_SECTOR);
    if (valid_sector == No_Valid_Sector) return  No_Valid_Sector;
    /* Get the variable start address in the valid sector */
    address = (u32)(T[index] + (u32)((valid_sector - 0x01) * SECTOR_SIZE));
    /* Get the variable end address in the valid sector */
    /* If the variable isn't the last one in the sector */
    if (index != max_varia-1)
    {
        variable_end_address = (u32)(T[index+1]) ;
        variable_end_address+= (u32)((valid_sector - 0x1) * SECTOR_SIZE);
    }
    /*variable is the last one in the sector, it occupies the remaining space memory in
      the sector: until SECTOR_END_ADDRESS) */

    else

    {
        variable_end_address = (u32)SECTOR1_END_ADDRESS;
        variable_end_address +=(u32)((valid_sector - 0x01) * SECTOR_SIZE);
    }
    do
    {   /* verify if address content is 0xFFFFFFFF ; if yes status = 0, if no status = 0xBADC*/
        status  = FlashVerifyWord(address, 0xFFFFFFFF);
        /* verify if (address + 4) content is 0xFFFFFFFF ; if yes status1 = 0, if no status1 = 0xBADC*/
        status1 = FlashVerifyWord(address + 4, 0xFFFFFFFF) ;
        /* verify if at (address + 4) content is 0x00000000; if yes status2 = 0,  if no status2 = 0xBADC*/
        status2 = FlashVerifyWord(address + 4, 0x00000000) ;
        if ((status == 0) &&(status1 == 0))
        {
            if (Data!= 0xFFFFFFFF)
            {
                /* address is a blank location so write the data */
                status3 = FlashProgramWord(address, Data);
                break;
            }
            else /* Data = 0xFFFFFFFF */
            {
                address = address + 4;
                if (address >= variable_end_address)
                {
                    status3 = VARIABLE_FULL;
                    break;
                }
                /* Write 0x00000000 at address + 4	*/
                status3 = FlashProgramWord(address, 0x00000000);
                break;
            }
        }
        /* 0xFFFFFFFF is a data */
        else if ((status == 0) && (status2 == 0))	 address = address + 8;
        else address = address + 4;
        if (address >= variable_end_address)
        {
            /* 0x80 is returned */
            status3 = VARIABLE_FULL;
            break;
        }
    } while (address < variable_end_address);  /* End do */
    return status3;

}  /* End WriteVerifyVariableFull */
/*******************************************************************************
* Routine:  EepromSectorTransfer
* Purpose:  Transfers data from full sector to an empty one.
* Input  :  - index:variable index in the array (0, 1, 2 ...)
*		    - T: Variables' Array
*			- data: 32 bit data to be written
* Output :   None.
* Return : 	 0 on success or error code
*******************************************************************************/
u16  EepromSectorTransfer(u8 index, u32 *T, u32 data)
{
    u32 Data;
    u8 OLD_SECTOR, valid_sector, i=0;
    u16 status = 0;
    u32  new_sector; /* The sector we are moving to	*/
    u32  old_sector; /* The sector we are moving from */
    /* Get active sector  (This is the sector we are moving from) */
    valid_sector = FindValidSector(READ_FROM_VALID_SECTOR);

    if (valid_sector == SECTOR2)
    {
        new_sector = SECTOR1_BASE_ADDRESS;  // move data to this sector
        old_sector = SECTOR2_BASE_ADDRESS;  // move data from this sector
        OLD_SECTOR = SECTOR2;
    }
    else if (valid_sector == SECTOR1)
    {
        new_sector = SECTOR2_BASE_ADDRESS; // move data to this sector
        old_sector = SECTOR1_BASE_ADDRESS; // move data from this sector
        OLD_SECTOR = SECTOR1;
    }
    else return  No_Valid_Sector;
    /* Mark new sector to receive */
    status = FlashProgramWord(new_sector,RECEIVE_DATA);
    /* if error, return error code */
    if (status !=0) return status;
    /* Transfer process */
    do
    {
        /* Write the variable data */
        if (i == index)
        {
            status = WriteVerifyVariableFull(index,T,data);
            if (status !=0) return status;
        }
        /* Transfer other variables last updates */
        else
        {
            /* Read the other last variable updates	*/
            Data = ReadVariable(i,T);
            if (Data != VARIABLE_EMPTY)	 status = WriteVerifyVariableFull(i,T,Data);
            if (status!=0) return status;
        }
        i++;
    } while (i < max_varia);
    /* Mark good and bad sectors */
    /* Mark old sector TRANSFER_COMPLETE before marking new sector VALID_SECTOR */
    status = FlashProgramWord(old_sector, TRANSFER_COMPLETE);
    /* Program routine returns 0 on success and Flash error code on failure.*/
    if (status!=0) return status;
    /* mark new sector VALID_SECTOR after marking old sector TRANSFER_COMPLETE*/
    status = FlashProgramWord(new_sector, VALID_SECTOR);
    /* Program routine returns 0 on success and Flash error code on failure.*/
    if (status!=0) return status;
    /* Erase the old sector */
    FlashSectorErase(0,OLD_SECTOR);
    /* Wait until the erase operation is complete */
    status =  FlashWait(FLASH_DELAY_5SEC);
    /* If the erase operation is successful, 0 is returned. If the erase operation is failed,
    a Flash error code is returned.*/
    return status;
} /* End EepromSectorTransfer */


/************************ (c) 2006  ST Microelectronics ******* END OF FILE ***/